-- TODO
-- Initialization of start color
-- Return of selected color

local _ -- garbage collector
local MAX_HUE = 359
local MAX_VALUE = 255

--gkmisc.ColorPicker = gkmisc.ColorPicker or {} -- doesn't seem to properly preserve the images like I hoped.
-- probably tied to the incomplete nature of the implementation of iup.Image or something else I'm missing
gkmisc.ColorPicker = {} -- didn't want to redo code below.  always recreate the table to force recreation of images

declare("ColorPicker", {
	ImageAttributes = {
		Palette = {l = 360, t = 256, fV = true, b = true},
		-- Drawn and refreshed images are too costly and cause memory leaks
-- 		Saturation = {l = 256, t = 15, st = "S", fH = false, fV = true, v = true, H = true, V = true, b = true},
-- 		Alpha = {l = 256, t = 15, st = "A", H = true, S = true, V = true, b = true},

		Sat2a = {ignore = true, l = 256, t = 15, st = "A", fV = false, v = true, H = 0, S = 0, V = 1, b = true},
		Sat2c = {ignore = true, l = 256, t = 15, st = "S", fV = true, v = true, H = 0, S = 0, V = 1, b = true},
		Alpha2 = {ignore = true, l = 256, t = 15, st = "A", fH = false, v = false, H = 0, S = 0, V = 1, b = true},
	},
	ImageMaps = {
		TriangleDown = { 
			-- For some reason only rotation = 270 doesn't distort this image,
			-- and it gives me the right facing arrow I want in addition to the down arrow
			{1,1,1,1,1,1,1,1,1},
			{0,1,1,1,1,1,1,1,0},
			{0,0,1,1,1,1,1,0,0},
			{0,0,0,1,1,1,0,0,0},
			{0,0,0,0,1,0,0,0,0},
		},
	},
})

local Hue, Sat, Val = 0, 1, 1 -- current values as set by clicking on the HV map or Saturation selector

local function ImageDef(w, h, d, R, G, B, A)
	-- TODO add assertion tests
	w = h and w or nil
	h = w and h or nil
	d = d or 3 -- 3 full alpha, 4 assigned alpha
	local t = {w = w, h = h, depth = d}
		
	function t:Clear() for y = 1, #self do self[y] = nil end end

	function t:GetPoint(x, y)
		local R = self[y + 1][x * 4 + 3]
		local G = self[y + 1][x * 4 + 2]
		local B = self[y + 1][x * 4 + 1]
		local A = self[y + 1][x * 4 + 4]
		return string.format("%d %d %d %d", R, G, B, A), R, G, B, A
	end

	function t:Init(R, G, B, A)
		R, G, B, A = R or 0, G or 0, B or 0, A or 255
		self:Clear()
		for y = 0, self.h - 1 do
			self[y + 1] = {}
			for x = 0, self.w - 1 do
				self:SetPoint(x, y, R, G, B, A)
			end
		end
	end

	function t:SetPoint(x, y, R, G, B, A)
		-- ?Add support for IUP color strings?
		if (0 <= x and x <= self.w - 1 and 0 <= y and y <= self.h - 1) then
			-- iup.image is formatted BGRA, reorder done here to maintain sanity of RGBA ordering
			for i, v in ipairs{B, G, R, A} do self[y + 1][x * 4 + i] = v end
		end
	end
	
	t:Init(R, G, B, A)
	
	return t
end

local function drawCircle(radius, x0, y0, R, G, B, A)
	x0 = x0 or radius
	y0 = y0 or radius
	R, G, B, A = R or 255, G or 255, B or 255, A or 255
	local x, y, r2
	local t = ImageDef(x0 * 2 + 1, x0 * 2 + 1, 4, nil, nil, nil, 0)
	
	-- Algorithm taken from https://en.wikipedia.org/wiki/Midpoint_circle_algorithm and modified for Lua
	x = radius;
    y = 0;
    err = 0;
	t:SetPoint(x0, y0, 0, 255, 0, 255)
	while (x >= y) do
        t:SetPoint(x0 + x, y0 + y, R, G, B, A);
        t:SetPoint(x0 + y, y0 + x, R, G, B, A);
        t:SetPoint(x0 - y, y0 + x, R, G, B, A);
        t:SetPoint(x0 - x, y0 + y, R, G, B, A);
        t:SetPoint(x0 - x, y0 - y, R, G, B, A);
        t:SetPoint(x0 - y, y0 - x, R, G, B, A);
        t:SetPoint(x0 + y, y0 - x, R, G, B, A);
        t:SetPoint(x0 + x, y0 - y, R, G, B, A);

        y = y + 1;
        err = err + 1 + 2*y;
        if (2*(err-x) + 1 > 0) then 
            x = x - 1;
            err = err + 1 - 2*x;
		end
	end
	return t
end

local function drawImageMap(im, R, G, B, A)
	-- assuming a rectangular map; currently only supports a single color
	local t = ImageDef(#im[1], #im, 4, nil, nil, nil, 0)
	
	for y, r in ipairs(im) do
		for x, v in ipairs(r) do
			if (v ~= 0) then
				t:SetPoint(x - 1, y - 1, R, G, B, A)
			end
		end
	end
	return t
end

declare("HSV2RGB", function(H, S, V)
	local C, X, H_, m
	local R, R_, G, G_, B, B_

	C = V * S
	H_ = H / 60
	X = C * (1 - math.abs(H_ % 2 - 1))

	if     (H_ >= 5) then R_, G_, B_ = C, 0, X
	elseif (H_ >= 4) then R_, G_, B_ = X, 0, C
	elseif (H_ >= 3) then R_, G_, B_ = 0, X, C
	elseif (H_ >= 2) then R_, G_, B_ = 0, C, X
	elseif (H_ >= 1) then R_, G_, B_ = X, C, 0
	elseif (H_ >= 0) then R_, G_, B_ = C, X, 0
	end
	m = V - C
	R, G, B = R_+m, G_+m, B_+m

	R = math.floor(R * 255)
	G = math.floor(G * 255)
	B = math.floor(B * 255)
	
	return string.format("%d %d %d", R, G, B), R, G, B
end)

local function makepalette(width, height, flipH, flipV, vertical, adj, Hset, Sset, Vset)
	--[[
	    width = <number>; set the width of the resultant image; by default this corresponds to the X-axis
		height = <number>; set the height of the resultant image; by default this corresponds to the Y-axis
		flipH = <boolean>; flip the image render direction over the Y-axis
	    flipV = <boolean>; flip the image render direction over the X-axis
	    vertical = <boolean>; swap the width and height variables; TODO can label image rotation do this reliably?
	    adj = "S"|"A"; "S" creates a saturation selector; "A" creates an alpha selector; nil creates HV map
		Hset = <number>; fixed Hue values
	    Sset = <number>; fixed Saturation value
	    Vset = <number>; fixed Value value
	]]

	width = width or MAX_HUE -- global X; width of the palette and color range for HV map
	height = height or MAX_VALUE
	
	if (vertical) then width, height = height, width end
	
	local H, H_, S, V
	local C, X, m
	local R, R_, G, G_, B, B_, A
	local t = ImageDef(width, height, adj == "A" and 4 or 3) -- 4 will respect alpha values
	local xx, yy -- couldn't think of a better more succint name to hold possibly adjusted x/y values in case of flip(H|V)

	height = height - 1 -- to maintain height when adjusting to 0 based indexing
	width = width - 1 -- to maintain width when adjusting to 0 based indexing
	
	
	for y = 0, height do
		V = Vset or (y / height)
		S = Sset or (adj == "S" and (y / height)) or 1
		A = (adj == "A" and math.floor(y * (height / 255))) or 255
		for x = 0, width do
			if (adj == "S" and not vertical) then S = Sset or (x / 255) end 
			H = Hset or (x / (width / 359))

			_, R, G, B = HSV2RGB(H, S, V)

			if (adj == "A" and not vertical) then A = math.floor(x * (width / 255)) end
			xx = x
			yy = y
			if (flipH) then xx = width - x end
			if (flipV) then yy = height - y end
			t:SetPoint(xx, yy, R, G, B, A) 
		end
	end
	
	return t
end

function ColorPicker:CreateImage(i, attr)
	attr = attr or self.ImageAttributes[i]
	local fl = i:sub(1,1)
	local w, h = attr.l, attr.t
	local idef, image
	-- TODO rework image dimensions and creation

	if (gkmisc.ColorPicker[i]) then
		idef = gkmisc.ColorPicker[i].idef
		image = gkmisc.ColorPicker[i].image
	else
		idef = makepalette(attr.l, attr.t, attr.fH, attr.fV, attr.v, attr.st, attr.H and (tonumber(attr.H) or Hue), attr.S and (tonumber(attr.S) or Sat), attr.V and (tonumber(attr.V) or Val))
		image = iup.image(idef)
		image.width = idef.w
		gkmisc.ColorPicker[i] = {idef = idef, image = image}
	end
	local label  = iup.label{title = i, image = image, size = string.format("%dx%d", idef.w, idef.h)}

	return label, idef, image
end

function ColorPicker:CreateImageAndCanvas(i, redraw)
	local attr = self.ImageAttributes[i]
	local idef
	local fl = i:sub(1,1)

	if (redraw) then
		iup.Detach(self[i])
		iup.Destroy(self[i])
	end

	self[i], idef = self:CreateImage(i)
	
	if (redraw) then
		iup.Append(self[i.."Box"], self[i])
	else
		self[i.."Canvas"] = iup.canvas{
			size = string.format("%dx%d", idef.w + 2, idef.h + 2), -- accounting for the border width
			expand = "NO", 
			border = attr.b and "ON" or "OFF", 
			attr = attr
		}
		self[i.."Box"] = iup.zbox{
			self[i],
			self[i.."Canvas"],
			all = "YES",
			alignment = "ACENTER",
		}
	end
end

function ColorPicker:BuildDialog()

	local w, h
	local idef -- used to hold imagedefs as they're created
	local iattr
	local tri_idef
	
	-- Triangle indicators for selectors
	if (gkmisc.ColorPicker.TriangleDown) then
		tri_idef = gkmisc.ColorPicker.TriangleDown.idef
		self.TriangleDown = gkmisc.ColorPicker.TriangleDown.image
	else
		local im = self.ImageMaps.TriangleDown
		tri_idef = drawImageMap(im, 200, 200, 200, 255)
		self.TriangleDown = iup.image(tri_idef)
		self.TriangleDown.width = tri_idef.w
		gkmisc.ColorPicker.TriangleDown = {idef = tri_idef, image = self.TriangleDown}
	end
	self.TriangleDown = iup.label{title = "TriangleD", image = self.TriangleDown, rotation = 0, cx = 2 - math.floor((tri_idef.w / 2) + 0.5), cy = 0, size = string.format("%dx%d", tri_idef.w, tri_idef.h)}

	if (gkmisc.ColorPicker.TriangleRight) then
		tri_idef = gkmisc.ColorPicker.TriangleRight.idef
		self.TriangleRight = gkmisc.ColorPicker.TriangleRight.image
	else
		self.TriangleRight = iup.image(tri_idef)
		self.TriangleRight.width = tri_idef.w
		gkmisc.ColorPicker.TriangleRight = {idef = tri_idef, image = self.TriangleRight}
	end
	self.TriangleRight = iup.label{title = "TriangleR", image = self.TriangleRight, rotation = 270, cx = 0, cy = 2 - math.floor((tri_idef.w / 2) + 0.5), size = string.format("%dx%d", tri_idef.h, tri_idef.w)}

	-- Circle indicator for HV map
	if (gkmisc.ColorPicker.Circle) then
		idef = gkmisc.ColorPicker.Circle.idef
		self.Circle = gkmisc.ColorPicker.Circle.image
	else
		idef = drawCircle(5)
		self.Circle = iup.image(idef)
		self.Circle.width = idef.w
		gkmisc.ColorPicker.Circle = {idef = idef, image = self.Circle}
	end
	self.Circle = iup.label{title = "Circle", image = self.Circle, cx = -math.floor((idef.w / 2) + 0.5)+2, cy = -math.floor((idef.w / 2) + 0.5)+2, size = string.format("%dx%d", idef.w, idef.w)}
	self.Circle.bgcolor = "200 200 200 175 &"

	-- HV map and other images
	for i, d in pairs(self.ImageAttributes) do 
		if (not d.ignore) then self:CreateImageAndCanvas(i) end
	end

	self.PaletteBox.cx = 0
	self.PaletteBox.cy = 0
	self.HVcbox = iup.cbox{self.PaletteBox, self.Circle, self.Triangle, size = self.PaletteCanvas.size}

	-- Saturation2 (uses multiple images and bgcolor instead of redrawing the selector)
	iattr = self.ImageAttributes.Sat2c
	self.Sat2a = self:CreateImage("Sat2a")
	self.Sat2c, idef = self:CreateImage("Sat2c")
	self.Sat2c.bgcolor = HSV2RGB(Hue, Sat, Val)
	self.S2Canvas = iup.canvas{
		size = string.format("%dx%d",  idef.w + 2, idef.h + 2), -- accounting for border width
		expand = "NO", 
		border = iattr.b and "ON" or "OFF", 
		attr = iattr
	}
	self.S2Box = iup.zbox{
		self.Sat2c,
		self.Sat2a,
		self.S2Canvas,
		all = "YES",
		alignment = "ACENTER",
		cx = tri_idef.h,
		cy = 0,
	}
	self.S2cbox = iup.cbox{
		self.S2Box,
		self.TriangleRight,
		size = string.format("%dx%d", idef.w + tri_idef.h + 2, idef.h + 2), -- would use TriangleRight.w, but that's not available yet
		border = "ON"
	}

	-- Alpha2 (uses bgcolor instead of redrawing the selector)
	iattr = self.ImageAttributes.Alpha2
	self.Alpha2, idef = self:CreateImage("Alpha2")
	self.Alpha2.bgcolor = HSV2RGB(Hue, Sat, Val)
	self.A2Canvas = iup.canvas{
		size = string.format("%dx%d",  idef.w + 2, idef.h + 2),  -- accounting for border width
		expand = "NO", 
		border = iattr.b and "ON" or "OFF", 
		attr = iattr
	}
	self.TriangleDown.cx = idef.w - math.floor((tri_idef.w / 2) + 0.5) + 1
	self.A2Box = iup.zbox{
		self.Alpha2,
		self.A2Canvas,
		all = "YES",
		alignment = "ACENTER",
		cx = 0,
		cy = tri_idef.h
	}
	self.A2cbox = iup.cbox{
		self.A2Box,
		self.TriangleDown,
		size = string.format("%dx%d", idef.w + 2, idef.h + tri_idef.h + 2) -- would use TriangleDown.h, but that's not available yet
	}

	-- Color Sample
	do
		local edge = 25
		self.Sample = iup.image(ImageDef(edge, edge, 4, 255, 255, 255, 255))
		self.Sample.width = edge
		self.Sample = iup.label{title = "Sample", image = self.Sample, size = string.format("%dx%d", edge, edge), bgcolor = HSV2RGB(Hue, Sat, Val)}
	end

	-- value display labels
	self.Hval = iup.label{title = 0, size = 50, expand = "NO"}
	self.Sval = iup.label{title = 1, size = 50, expand = "NO"}
	self.Vval = iup.label{title = 1, size = 50, expand = "NO"}

	self.Rval = iup.label{title = 255, size = 50, expand = "NO"}
	self.Gval = iup.label{title = 0, size = 50, expand = "NO"}
	self.Bval = iup.label{title = 0, size = 50, expand = "NO"}
	self.Aval = iup.label{title = 255, size = 50, expand = "NO"}

	self.Mem = iup.label{title = 0, size = 100, expand = "NO"}

	self.collect = iup.stationbutton{title = "Collect", action = function() collectgarbage() end}

	self.reload = iup.stationbutton{title = "Reload", action = function() ColorPicker:UnlinkImages(); ReloadInterface() end}

	-- Dialog layout
	self.d = iup.dialog{
		iup.vbox{
			iup.fill{size = 5},
			iup.zbox{
				iup.hbox{
					iup.vbox{
						self.HVcbox,
		-- 				self.PaletteBox,
		-- 				iup.hbox{iup.fill{}, self.AlphaBox,},
						iup.hbox{iup.fill{}, self.A2cbox,},
						gap = 10,
					},
		-- 			self.SaturationBox,
					iup.vbox{
						self.S2cbox,
						gap = 10,
					},
					gap = 10,
					expand = "NO",
				}, 
				self.Sample,
				all = "YES",
				alignment = "SE",
			},
			iup.pdasubsubsubframebg{
				iup.vbox{
					iup.hbox{
						iup.label{title = "H: "}, self.Hval,
						iup.label{title = "S: "}, self.Sval,
						iup.label{title = "V: "}, self.Vval,
						gap = 5,
					},
					iup.hbox{
						iup.label{title = "R: "}, self.Rval,
						iup.label{title = "G: "}, self.Gval,
						iup.label{title = "B: "}, self.Bval,
						iup.label{title = "A: "}, self.Aval,
						iup.fill{},
						gap = 5,
					},
					iup.hbox{
						iup.label{title = "Mem Usage: "},
						self.Mem,
						iup.fill{},
						self.collect,
						self.reload,
						gap = 10,
					},
					gap = 10,
				},
			},
			gap = 10,
			expand = "NO",
		},
		bgcolor = "0 0 0 0 *",
		topmost = "YES"
	}

	local function get_selector_value_from_canvas_pos(c, x, y, r1, r2)
		r1 = r1 or 0
		r2 = r2 or MAX_VALUE
		local l = c.attr.l - 1
		local xval = x - c.X - 1; 
		local yval = y - c.Y - 1;

		local val = xval

		if (c.attr.v) then
			if (c.attr.fV) then val = l - yval else val = yval end
		elseif (c.attr.fH) then val = l - xval 
		end

		val = val / (l / r2)
		if (val < r1) then val = r1 elseif (val > r2) then val = r2 end
		return val
	end

	function self.PaletteCanvas:button_cb(b, s, x, y, f)
		if (s == 1) then -- button press
			self.interact = true
		elseif (s == 0) then -- button release
			self:motion_cb(x, y, f)
			ColorPicker:UpdateSelectors(true)
			self.interact = false
		end
	end
	function self.PaletteCanvas:motion_cb(x, y, f)
		if (self.interact) then
			local l = self.attr.l - 1 -- adjusting for zero-based image index
			local t = self.attr.t - 1

			Hue = x - self.X - 1 -- adjusting for border
			if (self.attr.fH) then Hue = l - Hue end
			Hue = Hue / (l / MAX_HUE)
			if (Hue < 0) then Hue = 0 elseif (Hue > MAX_HUE) then Hue = MAX_HUE end

			Val = y - self.Y - 1
			if (self.attr.fV) then Val = t - Val end
			Val = Val / (t / 1)
			if (Val < 0) then Val = 0 elseif (Val > 1) then Val = 1 end

			ColorPicker.Hval.title = Hue
			ColorPicker.Vval.title = Val
			ColorPicker:UpdateSelectors()
			

			local circle = ColorPicker.Circle
			local cbw = 2 -- canvas border width
			local cix, ciy = self.X + cbw, self.Y + cbw -- canvas interior X/Y
			local halfcircle = math.floor((circle.w / 2) + 0.5)
			-- Canvas coordinates reported are absolute to their screen location, not relative to the canvas
			
			local newx = x - cix - halfcircle -- unknown
			if (newx <= cbw) then
				newx = -halfcircle + cbw
			elseif (self.w - halfcircle - cbw + 1<= newx) then -- unknown
				newx = self.w - halfcircle - cbw + 1
			end
			circle.cx = newx
			
			local newy = y - ciy - halfcircle-- unknown
			if (newy <= 1) then 
				newy = -halfcircle + cbw
			elseif (self.h - halfcircle - cbw + 1<= newy) then -- unknown
				newy = self.h - halfcircle - cbw + 1
			end
			circle.cy = newy
			ColorPicker.HVcbox.REFRESH = "YES"
		end
	end

	-- function self.SaturationCanvas:button_cb(b, s, x, y, f)
	-- 	if (s == 1) then -- button press
	-- 		self.interact = true
	-- 	elseif (s == 0) then -- button release
	-- 		self:motion_cb(x, y, f)
	-- 		self:UpdateSelectors(true, true)
	-- 		self.interact = false
	-- 	end
	-- end
	-- function self.SaturationCanvas:motion_cb(x, y, f)
	-- 	--TODO: remove dependence on flag variable and test <f>
	-- 	if (self.interact) then
	-- 		Sat = get_selector_value_from_canvas_pos(self, x, y, 0, 1)
	-- 		self.Sval.title = Sat
	-- 	end
	-- end

	function self.S2Canvas:button_cb(b, s, x, y, f)
		if (s == 1) then -- button press
			self.interact = true
		elseif (s == 0) then -- button release
			self:motion_cb(x, y, f)
			self.interact = false
		end
	end
	function self.S2Canvas:motion_cb(x, y, f)
		--TODO: remove dependence on flag variable and test <f>
		if (self.interact) then
			Sat = get_selector_value_from_canvas_pos(self, x, y, 0, 1)
			ColorPicker:UpdateSelectors()
			ColorPicker.Sval.title = Sat

			local tri = ColorPicker.TriangleRight
			local cbw = 2 -- canvas border width
			local ciy = self.Y + cbw -- canvas interior Y
			local halftri = math.floor((tri.w / 2) + 0.5) -- rotated image, use width for height
			-- Canvas coordinates reported are absolute to their screen location, not relative to the canvas

			local newy = y - ciy - halftri -- gap and fill; unknown
			if (newy <= 0) then 
				newy = -halftri
			elseif (self.h - halftri - cbw - 2 <= newy) then -- unknown
				newy = self.h - halftri - cbw - 2
			end
			tri.cy = newy
			ColorPicker.S2cbox.REFRESH = "YES"


		end
	end

	-- function self.AlphaCanvas:button_cb(b, s, x, y, f)
	-- 	if (s == 1) then -- button press
	-- 		self.interact = true
	-- 	elseif (s == 0) then -- button release
	-- 		self:motion_cb(x, y, f)
	-- 		self.interact = false
	-- 	end
	-- end
	-- function self.AlphaCanvas:motion_cb(x, y, f)
	-- 	--TODO: remove dependence on flag variable and test <f>
	-- 	if (self.interact) then
	-- 		self.Aval.title = get_selector_value_from_canvas_pos(self, x, y)
	-- 	end
	-- end

	function self.A2Canvas:button_cb(b, s, x, y, f)
		if (s == 1) then -- button press
			self.interact = true
		elseif (s == 0) then -- button release
			self:motion_cb(x, y, f)
			self.interact = false
		end
	end
	function self.A2Canvas:motion_cb(x, y, f)
		--TODO: remove dependence on flag variable and test <f>
		if (self.interact) then
			ColorPicker.Aval.title = get_selector_value_from_canvas_pos(self, x, y)
			local s, R, G, B = HSV2RGB(Hue, Sat, Val)
			ColorPicker.Sample.bgcolor = string.format("%d %d %d %d *", R, G, B, tonumber(ColorPicker.Aval.title))

			local tri = ColorPicker.TriangleDown
			local cbw = 2 -- canvas border width
			local cix, ciy = self.X + cbw, self.Y + cbw -- canvas interior X/Y
			local halftri = math.floor((tri.w / 2) + 0.5)
			-- Canvas coordinates reported are absolute to their screen location, not relative to the canvas

			local newx = x - cix - halftri + 1 -- unknown
			if (newx <= cbw) then
				newx = -halftri + cbw
			elseif (self.w - halftri - 1 <= newx) then -- unknown
				newx = self.w - halftri - 1
			end
			tri.cx = newx
			ColorPicker.A2cbox.REFRESH = "YES"
		end
	end

	function self.d:show_cb()
		ColorPicker.Timer = Timer()
		local function timerloop()
			ColorPicker.Mem.title = string.format("%.2fkB", collectgarbage("count"))
			ColorPicker.Timer:SetTimeout(1000, timerloop)
		end
		timerloop()
	end
	
	function self.d:close_cb()
		if (ColorPicker.Timer:IsActive()) then ColorPicker.Timer:Kill() end
	end
	
	function self:UpdateSelectors(redraw, isSatClick)

		-- Redrawing images is slow and suffers from a memory leak
	-- 	if (redraw) then
	-- 		--TODO: move to a standalone function and add support for individual image regneration instead of both at once
	-- 		for _, i in ipairs{"Alpha", (not isSatClick and "Saturation") or nil} do
	-- 			CreateImageAndCanvas(i, true)
	-- 		end
	-- 		iup.Refresh(self.d)
	-- 	end
		
		ColorPicker.Sat2a.bgcolor = HSV2RGB(Hue, 0, Val)
		ColorPicker.Sat2c.bgcolor = HSV2RGB(Hue, 1, Val)
		local s, R, G, B = HSV2RGB(Hue, Sat, Val)
		ColorPicker.Alpha2.bgcolor, ColorPicker.Rval.title, ColorPicker.Gval.title, ColorPicker.Bval.title = s, R, G, B
		ColorPicker.Sample.bgcolor = string.format("%d %d %d %d *", R, G, B, tonumber(ColorPicker.Aval.title))
	end

	self.d:map()
end

function ColorPicker:UnlinkImages()
	-- Doesn't seem to do what I hoped it would, or the approach is wrong
-- 	self.Palette.image = nil
-- 	self.TriangleDown.image = nil
-- 	self.TriangleRight.image = nil
-- 	self.Circle.image = nil
-- 	self.Sat2a.image = nil
-- 	self.Sat2c.image = nil
-- 	self.Alpha2.image = nil

end

ColorPicker:BuildDialog()

ShowDialog(ColorPicker.d)

